#!/usr/bin/python3
# -*- coding: utf-8 -*-
# ModuleName: Main
#-----------------------------------------------------------------------#
# Name:GimpSplasher                                                     #
# Version 3.0.7                                                         #
# Copyright (C) 2023 Ken Dotson - <lxgator[AT]gmail[DOT]com>            #
#                                                                       #
# This program is free software: you can redistribute it and/or modify  #
# it under the terms of the GNU General Public License as published by  #
# the Free Software Foundation, either version 3 of the License, or     #
# (at your option) any later version.                                   #
#                                                                       #
# This program is distributed in the hope that it will be useful,       #
# but WITHOUT ANY WARRANTY; without even the implied warranty of        #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the          #
# GNU General Public License for more details.                          #
#                                                                       #
# You should have received a copy of the GNU General Public License     #
# along with this program. If not, see <http://www.gnu.org/licenses/>.  #
#-----------------------------------------------------------------------#

import os
import sys
import mimetypes
import configparser
import gi

gi.require_version('Gtk', '3.0')
from gi.repository import Gtk, GdkPixbuf, Gio


class GimpSplasher(Gtk.Window):
    """Gtk+ 3 interface for GimpSplasher"""

    def __init__(self):
        """Initialize the window"""
        Gtk.Window.__init__(self, title='GimpSplasher')
        self.set_default_size(450, 475)
        self.set_icon_from_file('/usr/share/icons/hicolor/48x48/apps/gimpsplasher.png')
        self.set_position(Gtk.WindowPosition.CENTER)
        self.set_border_width(10)

        # read or create a config file
        #-------------------------------
        home_path = os.environ.get('HOME')
        self.conf_path = os.path.join(home_path, '.GimpSplasher.cfg')

        self.conf_file = configparser.ConfigParser()
        self.conf_file.read(self.conf_path)

        if os.path.isfile(self.conf_path):
            # read config file
            pics_path = self.conf_file.get('splash_folder', 'pics')
        else:
           # write a default config file
            pics_default = os.path.join(home_path, '/usr/share/gimpsplasher/splashes')
            self.conf_file.add_section('splash_folder')
            self.conf_file.set('splash_folder', 'pics', pics_default)
            self.conf_file.write(open(self.conf_path, 'w'))
            pics_path = pics_default

        # read pics path recursively
        #----------------------------
        self.pics_names_list(pics_path)

        # create a grid
        #---------------
        self.grid = Gtk.Grid(column_homogeneous=True,
                             column_spacing=10,
                             row_spacing=10)
        self.add(self.grid)

        # create a window with scroll bars
        #----------------------------------
        self.scrolledwindow = Gtk.ScrolledWindow()
        self.scrolledwindow.set_policy(Gtk.PolicyType.NEVER,
                                       Gtk.PolicyType.AUTOMATIC)

        # create a list store: pic, name, path
        #------------------------------------------
        self.liststore = Gtk.ListStore(GdkPixbuf.Pixbuf, str, str)
        for name, pic in zip(self.pics_name, self.pics_list):
            try:
                pxbf = GdkPixbuf.Pixbuf.new_from_file_at_scale(pic,
                                                               100,
                                                               100,
                                                               True)
                self.liststore.append([pxbf, name, pic])
            except:
                skip = "\033[1m[ SKIP ]\033[0m"
                print (skip + " Error reading file %s" % pic)

        # create the tree view
        #----------------------
        self.treeview = Gtk.TreeView(model=self.liststore)
        self.treeview.set_hexpand(True)
        self.treeview.set_vexpand(True)

        # create the columns for the tree view
        #--------------------------------------
        renderer_pixbuf = Gtk.CellRendererPixbuf()  # column for pic
        column_pixbuf = Gtk.TreeViewColumn('Images', renderer_pixbuf,
                                           pixbuf=0)
        column_pixbuf.set_alignment(0.5)
        self.treeview.append_column(column_pixbuf)

        renderer_text = Gtk.CellRendererText(weight=600)  # column for pics
        renderer_text.set_fixed_size(200, -1)
        column_text = Gtk.TreeViewColumn('Select One or Multiple Images to Apply', renderer_text, text=1)
        column_text.set_sort_column_id(1)
        column_text.set_alignment(0.5)
        self.treeview.append_column(column_text)

        self.scrolledwindow.add(self.treeview)
        self.grid.attach(self.scrolledwindow, 0, 0, 1, 1)

        self.treeview.connect('row-activated', self.on_row_activated)

         # create the buttons
        #--------------------
        icon = Gio.ThemedIcon(name="gtk-about")
        image = Gtk.Image.new_from_gicon(icon, Gtk.IconSize.BUTTON)
        
        self.buttonbox = Gtk.ButtonBox(homogeneous=False, spacing=10)
        self.buttonbox.set_layout(Gtk.ButtonBoxStyle.END)
        self.button_about = Gtk.Button.new_with_mnemonic("About")
        self.button_about.set_image(image) 
        self.button_about.connect('clicked', self.on_button_about_clicked)
        self.buttonbox.add(self.button_about)
        self.buttonbox.set_child_secondary(self.button_about,
                                           is_secondary=True)
                                           
        icon = Gio.ThemedIcon(name="gtk-preferences")
        image = Gtk.Image.new_from_gicon(icon, Gtk.IconSize.BUTTON)
        
        self.button_prefs = Gtk.Button.new_with_mnemonic("_Preferences")
        self.button_prefs.set_image(image)        
        self.button_prefs.connect('clicked', self.on_button_prefs_clicked)
        self.buttonbox.add(self.button_prefs)
        self.buttonbox.set_child_secondary(self.button_prefs,
                                           is_secondary=True)
                                           
        icon = Gio.ThemedIcon(name="gtk-close")
        image = Gtk.Image.new_from_gicon(icon, Gtk.IconSize.BUTTON)                                           

        self.button_close = Gtk.Button.new_with_mnemonic("_Close")
        self.button_close.set_image(image)         
        self.button_close.connect('clicked', self.on_button_close_clicked)
        self.buttonbox.add(self.button_close)
        self.buttonbox.set_child_secondary(self.button_close,
                                           is_secondary=True)
                                           
        icon = Gio.ThemedIcon(name="gtk-apply")
        image = Gtk.Image.new_from_gicon(icon, Gtk.IconSize.BUTTON)   
        
        self.button_apply = Gtk.Button.new_with_mnemonic("_Apply")
        self.button_apply.set_image(image)         
        self.button_apply.connect('clicked', self.on_button_apply_clicked)
        self.buttonbox.add(self.button_apply)


        self.grid.attach_next_to(self.buttonbox, self.scrolledwindow,
                                Gtk.PositionType.BOTTOM, 1, 1)

    # open image on double-click
    #----------------------------
    def on_row_activated(self, widget, path, column):
        """Preview image when double-clicked"""
        selection = self.treeview.get_selection()
        model, treeiter = selection.get_selected()
        if treeiter is not None:
            name, pic = model[treeiter][1:]
            img = ViewPic(name, pic)

    # function to read the images folder
    #--------------------------------------
    def pics_names_list(self, pics_path):
        """Scan the splash folder recursively"""
        self.pics_list = []
        self.pics_name = []
        # read recursively
        for root, dirs, files in os.walk(pics_path):
            for fn in files:
                if self.mmtype(fn):
                    self.pics_name.append(fn)
                    f = os.path.join(root, fn)
                    self.pics_list.append(f)

    # filter file by mime type
    #--------------------------
    def mmtype(self, filein):
        """Select images based on mime types"""
        mime_list = ['image/png',
                     'image/jpeg',
                     'image/gif',
                     'image/tiff',
                     'image/bmp']
        mime = mimetypes.guess_type(filein)[0]
        if mime in mime_list:
            return True

    # callback for button Apply
    #---------------------------
    def on_button_apply_clicked(self, widget):
        """Apply the selected wallpaper"""
        # apply the wallpaper
        selection = self.treeview.get_selection()
        model, treeiter = selection.get_selected()
        if treeiter is not None:
            name, pic = model[treeiter][1:]
        if os.path.exists(os.path.expanduser("~/.config/GIMP/2.10/splashes")):
            dest_dir = os.path.expanduser("~/.config/GIMP/2.10/splashes")
        else:
            dest_dir = os.path.expanduser("~/.config/GIMP/3.0/splashes")
        if os.fork() == 0:
            os.execvp("cp", ["cp", pic, dest_dir])
        else:
            os.popen("python3 /usr/share/gimpsplasher/add_image.py 2>/dev/null")        
            def on_button_about_clicked(self, widget):  
                dialog.destroy()    
                Gtk.main_quit()


    # callback for button Cancel
    #---------------------------
    def on_button_close_clicked(self, button):
        print("")
        Gtk.main_quit()


    # callback for button About
    #---------------------------
    def on_button_about_clicked(self, widget):
        """Show about dialog"""
        about = Gtk.AboutDialog()
        lic = """
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.

"""
        about.set_position(Gtk.WindowPosition.CENTER)
        about.set_program_name('GimpSplasher')
        about.set_version('3.0.8')
        about.set_copyright('© 2024 Ken Dotson lxgator@gmail.com')
        self.set_icon_from_file('/usr/share/icons/hicolor/48x48/apps/gimpsplasher.png')
        about.set_icon_from_file('/usr/share/icons/hicolor/48x48/apps/gimpsplasher.png')
        about.set_logo_icon_name('gimpsplasher')
        about.set_license(lic)
        about.set_website('http://www.pclosdebian.com/')
        about.set_comments('Python program to set splash images in Gimp')
        about.set_authors(['© 2024 Ken Dotson lxgator@gmail.com'])
        about.connect('response', self.on_about_closed)
        about.show()

    # close about dialog

    #--------------------
    def on_about_closed(self, widget, parameter):
        """Destroy about dialog"""
        widget.destroy()

    # callback for button Preferences
    #---------------------------------
    def on_button_prefs_clicked(self, widget):
        """Show preferences dialog and update splash folder"""
        dialog = PrefsDialog(self)
        response = dialog.run()
        if response == Gtk.ResponseType.OK:
            pics_new_path = dialog.entry_dir.get_text()
            if pics_new_path != '':
                # read pics path
                self.pics_names_list(pics_new_path)
                # if not empty...
                if len(self.pics_list) != 0:
                    # ...update pics folder in config file...
                    self.conf_file.set('splash_folder', 'pics', pics_new_path)
                    self.conf_file.write(open(self.conf_path, 'w'))
                    # ... and reload tree view
                    self.liststore = Gtk.ListStore(GdkPixbuf.Pixbuf, str, str)
                    for name, pic in zip(self.pics_name, self.pics_list):
                        try:
                            pxbf = GdkPixbuf.Pixbuf.new_from_file_at_scale(pic,
                                                                           100,
                                                                           100,
                                                                           True)
                            self.liststore.append([pxbf, name, pic])
                        except:
                            skip = "\033[1m[ SKIP ]\033[0m"
                            print (skip + " Error reading file %s" % pic)
                        self.treeview.set_model(model=self.liststore)
        dialog.destroy()


class PrefsDialog(Gtk.Dialog):
    """Preferences dialog"""

    def __init__(self, parent):
        """Initialize the window"""
        Gtk.Dialog.__init__(self, 'Select splash folder', parent, 0,
                            (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                             Gtk.STOCK_OK, Gtk.ResponseType.OK))
        self.set_default_size(300, 100)
        self.set_icon_from_file('/usr/share/icons/hicolor/48x48/apps/gimpsplasher.png')
        self.set_modal(True)

        self.grid = Gtk.Grid(column_spacing=5, row_spacing=5)

        self.entry_dir = Gtk.Entry()
        self.entry_dir.set_hexpand(True)
        self.grid.add(self.entry_dir)

        self.button_open = Gtk.Button.new_with_mnemonic("_Open")
        self.button_open.connect('clicked', self.on_button_open_clicked)
        self.grid.attach(self.button_open, 1, 0, 1, 1)

        box = self.get_content_area()
        box.add(self.grid)
        self.set_icon_from_file('/usr/share/icons/hicolor/48x48/apps/gimpsplasher.png')
        self.show_all()

    def on_button_open_clicked(self, widget):
        """Open a file chooser to select the splash folder"""
        dialog = Gtk.FileChooserDialog('Select splash folder', self,
                                       Gtk.FileChooserAction.SELECT_FOLDER,
                                       (Gtk.STOCK_CANCEL,
                                        Gtk.ResponseType.CANCEL,
                                        Gtk.STOCK_OPEN,
                                        Gtk.ResponseType.OK))
        response = dialog.run()
        if response == Gtk.ResponseType.OK:
            dname = dialog.get_filename()
            self.entry_dir.set_text(dname)
        dialog.destroy()


class ViewPic(Gtk.Window):
    """View selected pic on double-click"""

    def __init__(self, name, pic):
        """Initialize the window"""
        Gtk.Window.__init__(self)
        self.set_title(name)
        self.set_resizable(False)
        self.set_icon_from_file('/usr/share/icons/hicolor/48x48/apps/gimpsplasher.png')
        self.set_position(Gtk.WindowPosition.CENTER)
        self.set_border_width(10)

        self.grid = Gtk.Grid()
        self.add(self.grid)

        self.image = Gtk.Image()
        self.pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_scale(pic,
                                                            600,
                                                            600,
                                                            True)
        self.image.set_from_pixbuf(self.pixbuf)

        self.grid.add(self.image)
        self.show_all()


def main():
    """Show the window"""
    win = GimpSplasher()
    win.connect('delete-event', Gtk.main_quit)
    win.show_all()
    Gtk.main()
    return 0
if __name__ == '__main__':
    main()
